@@ -1355,7 +1355,7 @@ pub(crate) fn fstatfs64<Fd: AsFd>(fd: Fd) -> Result<Statfs64, Errno> {
     Ok(Statfs64(unsafe { dst.assume_init() }))
 }
 
-/// Safe wrapper for epoll_ctl with detailed error handling
+/// Safe wrapper for epoll_ctl with detailed error handling.
 pub fn epoll_ctl_safe<E: AsFd>(
     epoll: &E,
     fd: RawFd,
@@ -1397,6 +1397,24 @@ pub fn epoll_ctl_safe<E: AsFd>(
     }
 }
 
+/// Safe wrapper for epoll_ctl_mod with detailed error handling.
+pub fn epoll_ctl_mod_safe<E: AsFd>(
+    epoll: &E,
+    fd: RawFd,
+    mut event: libc::epoll_event,
+) -> Result<(), Errno> {
+    // SAFETY: In libc we trust.
+    Errno::result(unsafe {
+        libc::epoll_ctl(
+            epoll.as_fd().as_raw_fd(),
+            EpollOp::EpollCtlMod as libc::c_int,
+            fd,
+            &mut event,
+        )
+    })
+    .map(drop)
+}
+
 const EPIOCSPARAMS: IoctlRequest = 0x40088a01;
 const EPIOCGPARAMS: IoctlRequest = 0x80088a02u32 as IoctlRequest;
 
