@@ -43,9 +43,9 @@ use crate::{
     fs::{duprand, FileMap, FileMapEntry, FileType},
     fstatx, info,
     path::{XPath, XPathBuf},
-    pool::PidFdMap,
     proc::proc_mmap_min_addr,
     sysinfo::RandTimer,
+    workers::PidFdMap,
 };
 
 // Convenience type to list {io,pr}ctls with their names.
@@ -79,11 +79,20 @@ pub const ENV_NPROC: &str = "SYD_NPROC";
 pub const ENV_NPROC_MAX: &str = "SYD_NPROC_MAX";
 /// The environment variable to read the pid filename from.
 pub const ENV_PID_FN: &str = "SYD_PID_FN";
+/// The environment variable to force randomized fds.
+///
+/// Setting this environment variable is equivalent to setting trace/force_rand_fd:1.
+pub const ENV_FORCE_RANDFD: &str = "SYD_FORCE_RANDFD";
 /// The environment variable to force TTY output.
 pub const ENV_FORCE_TTY: &str = "SYD_FORCE_TTY";
 /// The environment variable to quiet TTY output.
 pub const ENV_QUIET_TTY: &str = "SYD_QUIET_TTY";
 
+/// The environment variable to hold the IPC epoll fd (used internally).
+pub const ENV_IPC_POLL_FD: &str = "SYD_IPC_POLL_FD";
+/// The environment variable to hold the IPC UNIX socket (used internally).
+pub const ENV_IPC_UNIX_FD: &str = "SYD_IPC_UNIX_FD";
+
 /// The environment variable to read the default external address from.
 pub const ENV_PROXY_HOST: &str = "SYD_PROXY_HOST";
 /// The environment variable to read the default external port from.
@@ -145,6 +154,9 @@ pub static VERSION: Lazy<&'static str> = Lazy::new(|| {
 /// Api version of the configuration.
 pub const API_VERSION: &str = env!("CARGO_PKG_VERSION_MAJOR");
 
+/// Api version for the IPC service.
+pub const IPC_VERSION: &str = "1";
+
 /// File format marker for Crypt sandboxing.
 pub const CRYPT_MAGIC: &[u8] = &[0x7F, b'S', b'Y', b'D', 3];
 
@@ -258,6 +270,10 @@ pub const MON_STACK_SIZE: usize = 96 * 1024;
 /// Defaults to 256k.
 pub const INT_STACK_SIZE: usize = 256 * 1024;
 
+/// Stack size for the IPC thread.
+/// Defaults to 256k.
+pub const IPC_STACK_SIZE: usize = 256 * 1024;
+
 /// Stack size for the AES encryption threads.
 /// Defaults to 96k.
 pub const AES_STACK_SIZE: usize = 96 * 1024;
@@ -535,9 +551,12 @@ pub const PROFILE_CORE: &[&str] = &["trace/allow_unsafe_prlimit:true"];
 
 /// Debug sandboxing profile.
 pub const PROFILE_DEBUG: &[&str] = &[
-    "trace/allow_unsafe_prlimit:true",  // enable coredumps.
-    "trace/allow_unsafe_ptrace:true",   // strace -f syd
-    "trace/allow_unsafe_dumpable:true", // PR_SET_DUMPABLE!
+    "trace/allow_unsafe_dumpable:true",   // PR_SET_DUMPABLE!
+    "trace/allow_unsafe_magiclinks:true", // allow /proc/$pid/mem access.
+    "trace/allow_unsafe_memory:true",     // allow W^X memory.
+    "trace/allow_unsafe_prctl:true",      // allow changing process name.
+    "trace/allow_unsafe_prlimit:true",    // enable coredumps.
+    "trace/allow_unsafe_ptrace:true",     // strace -f syd
 ];
 
 /// No memory restrictions sandboxing profile.
@@ -546,6 +565,16 @@ pub const PROFILE_NOMEM: &[&str] = &["trace/allow_unsafe_memory:true"];
 /// No PIE sandboxing profile.
 pub const PROFILE_NOPIE: &[&str] = &["trace/allow_unsafe_nopie:true"];
 
+/// Firefox sandboxing profile.
+/// `syd --profile ff`
+pub const PROFILE_FIREFOX: &[&str] = &[
+    "filter/read+!proc/name",             // for PR_SET_NAME.
+    "trace/allow_unsafe_filename:true",   // Allow profile directories with curly braces
+    "trace/allow_unsafe_libc:true",       // Allow preloading libmozsandbox.so
+    "trace/allow_unsafe_memory:true",     // W^X memory required for JIT
+    "trace/allow_unsafe_page_cache:true", // mincore(2) may be needed for DRM
+];
+
 /// Readonly sandboxing profile.
 /// `syd --profile ro`
 pub const PROFILE_READONLY: &[&str] = &["deny/wrset+/***"];
@@ -582,7 +611,6 @@ pub const PROFILE_LANDLOCK: &[&str] = &[
     // /proc is always allowed.
     "allow/lock/read,readdir+/run",
     "allow/lock/read,readdir,exec+/sbin",
-    "allow/lock/read,readdir+/sys",
     "allow/lock/read,readdir,exec+/usr",
     "allow/lock/read,readdir+/var",
     "allow/lock/read,write+/dev/full",
@@ -601,6 +629,7 @@ pub const PROFILE_PALUDIS: &[&str] = &[
     "include_profile linux",
     "include_profile tty",
     "default/lock:warn", // set Landlock compat-level to best-effort when stacked with Landlock profile.
+    "allow/lock/read,readdir+/sys", // Relax Landlock to allow /sys when stacked with Landlock profile.
     "name/host:localhost", // only effective when combined with unshare/uts:1
     "lock:exec",         // esandbox
     "log/verbose:false", // verbose logging is intended for malware analysis.
@@ -621,6 +650,7 @@ pub const PROFILE_PALUDIS: &[&str] = &[
     "trace/allow_unsafe_nomseal:true", // mseal(2) is a mitigation against compromised Syd.
     "trace/allow_unsafe_nopie:true",
     "trace/allow_unsafe_open_path:true",
+    "trace/allow_unsafe_page_cache:true",
     "trace/allow_unsafe_perf:true",
     "trace/allow_unsafe_ptrace:true",
     "trace/allow_unsafe_prctl:true", // perl tests want to set process name.
@@ -1115,6 +1145,9 @@ pub const NICE_SYSCALLS: &[&str] = &[
     "setpriority",
 ];
 
+/// The list of page cache system calls.
+pub const PAGE_CACHE_SYSCALLS: &[&str] = &["cachestat", "mincore"];
+
 /// The list of perf/debugging system calls.
 pub const PERF_SYSCALLS: &[&str] = &[
     "lookup_dcookie",
@@ -1125,7 +1158,31 @@ pub const PERF_SYSCALLS: &[&str] = &[
 ];
 
 /// The list of ptrace system calls.
-pub const PTRACE_SYSCALLS: &[&str] = &["kcmp", "ptrace", "process_vm_readv", "process_vm_writev"];
+pub const PTRACE_SYSCALLS: &[&str] = &[
+    "kcmp",
+    "pidfd_getfd",
+    "ptrace",
+    "process_madvise",
+    "process_vm_readv",
+    "process_vm_writev",
+];
+
+/// The list of system calls which are of epoll family.
+pub const EPOLL_SYSCALLS: &[&str] = &[
+    // SAFETY: epoll fd is created once at startup,
+    // before the seccomp filters have been loaded.
+    // "epoll_create",
+    // "epoll_create1",
+    "epoll_ctl",
+    "epoll_ctl_old",
+    "epoll_wait",
+    "epoll_wait_old",
+    "epoll_pwait",
+    "epoll_pwait2",
+];
+
+/// The list of system calls which are of UTS family.
+pub const UTS_SYSCALLS: &[&str] = &["setdomainname", "sethostname"];
 
 /// The list of system calls which are of the getid family.
 pub const GET_ID_SYSCALLS: &[&str] = &[
@@ -1325,9 +1382,10 @@ pub const DEAD_SYSCALLS: &[&str] = &[
     "lsm_list_modules",
     "mbind",
     "migrate_pages",
-    "mincore",
-    "mount_setattr",
     "move_pages",
+    // As of 3.35.2, mincore may be enabled using
+    // trace/allow_unsafe_page_cache:1 at startup.
+    //"mincore",
     "name_to_handle_at",
     "nfsservctl",
     "open_tree",
@@ -1474,7 +1532,7 @@ pub const SAFE_SYSCALLS: &[&str] = &[
     //"bpf", restrictions applied, see setup_seccomp.
     "breakpoint", // arm
     //"brk",
-    "cachestat",  // fd-only.
+    //"cachestat", // moved to page_cache syscalls as of 3.35.2.
     "cacheflush", // arm
     "capget",
     "capset",
@@ -1654,7 +1712,7 @@ pub const SAFE_SYSCALLS: &[&str] = &[
     "pause",
     //"perf_event_open", restrictions applied see load_seccomp_parent and setup_seccomp.
     "personality", // restrictions applied in load_seccomp_parent.
-    "pidfd_getfd",
+    //"pidfd_getfd", added to ptrace set as of 3.35.2.
     //"pidfd_open", restrictions applied, see setup_seccomp.
     "pidfd_send_signal",
     "pipe",
@@ -1671,7 +1729,7 @@ pub const SAFE_SYSCALLS: &[&str] = &[
     "preadv",
     "preadv2",
     //"prlimit64",
-    "process_madvise",
+    //"process_madvise", added to ptrace set as of 3.35.2.
     "process_mrelease",
     //process_vm_readv // restrictions applied, see setup_seccomp.
     //process_vm_writev // ditto.
@@ -1745,7 +1803,7 @@ pub const SAFE_SYSCALLS: &[&str] = &[
     "set_robust_list",
     "set_thread_area",
     "set_tid_address",
-    "setdomainname",
+    //"setdomainname", added to uts set as of 3.35.2.
     //"setfsgid",
     //"setfsgid32",
     //"setfsuid",
@@ -1754,7 +1812,7 @@ pub const SAFE_SYSCALLS: &[&str] = &[
     //"setgid32",
     //"setgroups",
     //"setgroups32",
-    "sethostname",
+    //"sethostname", added to uts set as of 3.35.2.
     "setitimer",
     "setns", // restrictions applied in load_seccomp_parent.
     "setpgid",
@@ -1887,7 +1945,7 @@ pub const EMU_SYSCALLS: &[&str] = &[
     "gettid",
     "getxattrat",
     "inotify_add_watch",
-    //"kcmp", only KCMP_VM is allowed.
+    //"kcmp", only KCMP_VM and KCMP_FILE are allowed.
     "landlock_create_ruleset",
     "landlock_restrict_self",
     "lgetxattr",
@@ -1899,7 +1957,7 @@ pub const EMU_SYSCALLS: &[&str] = &[
     "lsetxattr",
     "lstat",
     "madvise",
-    "memfd_create",
+    //"memfd_create", protected with syscall argument cookies.
     "mlock",
     "mmap",
     "mmap2",
@@ -1913,6 +1971,7 @@ pub const EMU_SYSCALLS: &[&str] = &[
     "pidfd_send_signal",
     "pipe2",
     "prctl",
+    "process_mrelease", // Used by kill action when signal is SIGKILL.
     "process_vm_readv",
     "process_vm_writev",
     "read",
@@ -1956,12 +2015,11 @@ pub const EMU_SYSCALLS: &[&str] = &[
     "mkdirat",
     "mknodat",
     "newfstatat",
-    "openat2",
+    //"openat2", protected with syscall argument cookies.
     "readlinkat",
-    "renameat",
-    "renameat2",
+    //"renameat2", protected with syscall argument cookies.
     "statx",
-    "unlinkat",
+    //"unlinkat", protected with syscall argument cookies.
     "utimensat",
 ];
 
@@ -1997,6 +2055,35 @@ pub const INT_SYSCALLS: &[&str] = &[
 /// Fcntl operations allowed for Interrupter thread.
 pub const INT_FCNTL_OPS: &[u64] = &[libc::F_GETFD as u64];
 
+/// System calls allowed for the IPC thread.
+pub const IPC_SYSCALLS: &[&str] = &[
+    "close",
+    "exit",
+    // "futex", See: FUTEX_SYSCALLS for futex family.
+    "getrandom",
+    "accept4",
+    "recvfrom",
+    "write",
+    // In case sandbox is locked through IPC,
+    // we need mmap family.
+    "madvise",
+    "mmap",
+    "mmap2",
+    "mprotect",
+    "mremap",
+    "mseal",
+    "munmap",
+    //
+    "restart_syscall",
+    "rt_sigprocmask",
+    "sched_yield",
+    "sigaltstack", // rare but necessary.
+    "sigprocmask", // ditto.
+];
+
+/// Fcntl operations allowed for IPC thread.
+pub const IPC_FCNTL_OPS: &[u64] = &[libc::F_GETFD as u64];
+
 /// System calls allowed for AES threads.
 pub const AES_SYSCALLS: &[&str] = &[
     "_llseek",
@@ -2097,6 +2184,7 @@ pub const MAIN_SYSCALLS: &[&str] = &[
     "pidfd_open",
     "pidfd_getfd",
     "pidfd_send_signal",
+    "process_mrelease",  // Used by kill action when signal is SIGKILL.
     "process_vm_readv",  // needed to get AT_{RANDOM,SECURE}.
     "process_vm_writev", // needed to set AT_SECURE.
     "ptrace",
@@ -2122,7 +2210,7 @@ pub const MAIN_SYSCALLS: &[&str] = &[
     // fd-calls
     "faccessat2",
     "newfstatat",
-    "openat2",
+    //"openat2", protected with syscall argument cookies.
     "readlinkat",
 ];
 
@@ -2367,10 +2455,12 @@ pub fn proc_init() -> Result<(), Errno> {
     // SAFETY: fd_root is a valid fd for process lifetime.
     let fd_root = unsafe { BorrowedFd::borrow_raw(fd_root) };
 
+    // Note, we may call getdents64(2) on /proc for PID sandboxing,
+    // therefore we open it without O_PATH here.
     let fd_proc = nix::fcntl::openat(
         fd_root,
         XPath::from_bytes(b"proc"),
-        OFlag::O_PATH | OFlag::O_DIRECTORY | OFlag::O_CLOEXEC,
+        OFlag::O_DIRECTORY | OFlag::O_CLOEXEC,
         Mode::empty(),
     )?;
     let id_proc = fstatx(&fd_proc, mask).map(|stx| stx.stx_mnt_id)?;
