@@ -12,6 +12,8 @@ use std::{
     os::fd::{AsFd, AsRawFd, BorrowedFd, FromRawFd, OwnedFd, RawFd},
 };
 
+#[allow(clippy::disallowed_types)]
+use ahash::{AHasher, RandomState};
 use constant_time_eq::constant_time_eq;
 use crc::{Crc, CRC_32_ISO_HDLC, CRC_64_ECMA_182};
 use hex::{DisplayHex, FromHex};
@@ -26,6 +28,7 @@ use nix::{
     },
     unistd::{lseek64, pipe2, read, write, Whence},
 };
+use procfs::{crypto as proc_crypto, SelfTest, Type};
 use secure_string::SecureBytes;
 use sha1::Sha1;
 use sha3::{Digest, Sha3_256, Sha3_384, Sha3_512};
@@ -34,7 +37,7 @@ use crate::{
     config::*,
     err::SydResult,
     fs::{
-        create_memfd_raw, fillrandom, mkstempat, retry_on_eintr, set_append, set_nonblock,
+        create_memfd, fillrandom, mkstempat, retry_on_eintr, set_append, set_nonblock,
         MFD_ALLOW_SEALING, MFD_NOEXEC_SEAL,
     },
     path::XPath,
@@ -388,6 +391,52 @@ pub fn hash<R: Read>(mut reader: R, func: HashAlgorithm) -> SydResult<Vec<u8>> {
     Ok(digest)
 }
 
+/// Returns a concise summary of hmac(sha256) shash support in the kernel.
+pub fn hmac_sha256_info() -> String {
+    match proc_crypto() {
+        Err(e) => format!("HMAC‐SHA256: failed to read /proc/crypto: {e}!"),
+        Ok(table) => {
+            if let Some(blocks) = table.crypto_blocks.get("hmac(sha256)") {
+                for block in blocks {
+                    if let Type::Shash(sh) = &block.crypto_type {
+                        let selftest = match block.self_test {
+                            SelfTest::Passed => "passed",
+                            SelfTest::Unknown => "unknown",
+                        };
+                        let internal = if block.internal {
+                            "in‐kernel"
+                        } else {
+                            "external"
+                        };
+                        let fips = if block.fips_enabled {
+                            "FIPS"
+                        } else {
+                            "no‐FIPS"
+                        };
+
+                        return format!(
+                            "HMAC‐SHA256: Secure hash is supported via '{}' driver; \
+module '{}'; prio {}; refcnt {}; \
+self‐test: {}; {}; {}; \
+blocksize {}B; digestsize {}B.",
+                            block.driver,
+                            block.module,
+                            block.priority,
+                            block.ref_count,
+                            selftest,
+                            internal,
+                            fips,
+                            sh.block_size,
+                            sh.digest_size,
+                        );
+                    }
+                }
+            }
+            "HMAC‐SHA256: Secure hash is unsupported!".to_string()
+        }
+    }
+}
+
 /// Sets up the HMAC-SHA256 authentication using the Kernel crypto API.
 ///
 /// # Arguments
@@ -460,14 +509,14 @@ pub fn hmac_sha256_feed<S: AsRawFd>(sock: &S, chunk: &[u8], more: bool) -> Resul
 }
 
 /// Finishes the HMAC-SHA256 authentication and reads authentication tag.
-pub fn hmac_sha256_fini<S: AsRawFd>(sock: &S) -> Result<SecureBytes, Errno> {
+pub fn hmac_sha256_fini<Fd: AsFd>(sock: Fd) -> Result<SecureBytes, Errno> {
     let mut data = SecureBytes::new(vec![0u8; SHA256_DIGEST_SIZE]);
     let buf = data.unsecure_mut();
 
     let mut nread = 0;
     while nread < SHA256_DIGEST_SIZE {
         #[allow(clippy::arithmetic_side_effects)]
-        match read(sock.as_raw_fd(), &mut buf[nread..]) {
+        match read(&sock, &mut buf[nread..]) {
             Ok(0) => return Err(Errno::EINVAL),
             Ok(n) => nread += n,
             Err(Errno::EINTR) => continue,
@@ -478,6 +527,57 @@ pub fn hmac_sha256_fini<S: AsRawFd>(sock: &S) -> Result<SecureBytes, Errno> {
     Ok(data)
 }
 
+/// Returns a concise summary of ctr(aes) skcipher support in the kernel.
+pub fn aes_ctr_info() -> String {
+    match proc_crypto() {
+        Err(e) => format!("AES-CTR: failed to read /proc/crypto: {e}!"),
+        Ok(table) => {
+            if let Some(blocks) = table.crypto_blocks.get("ctr(aes)") {
+                for block in blocks {
+                    if let Type::Skcipher(sk) = &block.crypto_type {
+                        let selftest = match block.self_test {
+                            SelfTest::Passed => "passed",
+                            SelfTest::Unknown => "unknown",
+                        };
+                        let internal = if block.internal {
+                            "in‐kernel"
+                        } else {
+                            "external"
+                        };
+                        let fips = if block.fips_enabled {
+                            "FIPS"
+                        } else {
+                            "no‐FIPS"
+                        };
+                        let async_cap = if sk.async_capable { "async" } else { "sync" };
+
+                        return format!(
+                            "AES‐CTR: Symmetric‐key cipher is supported via '{}' driver; \
+module '{}'; prio {}; refcnt {}; \
+self‐test: {}; {}; {}; {}; \
+key {}–{}B; iv {}B; chunk {}B; walk {}B.",
+                            block.driver,
+                            block.module,
+                            block.priority,
+                            block.ref_count,
+                            selftest,
+                            internal,
+                            fips,
+                            async_cap,
+                            sk.min_key_size,
+                            sk.max_key_size,
+                            sk.iv_size,
+                            sk.chunk_size,
+                            sk.walk_size,
+                        );
+                    }
+                }
+            }
+            "AES‐CTR: Symmetric‐key cipher is unsupported!".to_string()
+        }
+    }
+}
+
 /// Sets up the AES-CTR encryption/decryption using the Kernel crypto API.
 pub fn aes_ctr_setup(key: &Key) -> Result<OwnedFd, Errno> {
     // Create the socket for the AF_ALG interface
@@ -569,14 +669,14 @@ pub fn aes_ctr_dec<S: AsRawFd>(
 }
 
 /// Finishes the AES-CTR {en,de}cryption and reads the {de,en}crypted data.
-pub fn aes_ctr_fini<S: AsRawFd>(sock: &S, size: usize) -> Result<SecureBytes, Errno> {
+pub fn aes_ctr_fini<Fd: AsFd>(sock: Fd, size: usize) -> Result<SecureBytes, Errno> {
     let mut data = SecureBytes::new(vec![0u8; size]);
     let buf = data.unsecure_mut();
 
     let mut nread = 0;
     while nread < size {
         #[allow(clippy::arithmetic_side_effects)]
-        match read(sock.as_raw_fd(), &mut buf[nread..]) {
+        match read(&sock, &mut buf[nread..]) {
             Ok(0) => return Err(Errno::EINVAL),
             Ok(n) => nread += n,
             Err(Errno::EINTR) => continue,
@@ -590,19 +690,19 @@ pub fn aes_ctr_fini<S: AsRawFd>(sock: &S, size: usize) -> Result<SecureBytes, Er
 /// Decrypt the given file into a temporary fd with zero-copy.
 #[allow(clippy::cognitive_complexity)]
 #[allow(clippy::type_complexity)]
-pub fn aes_ctr_tmp<F: AsFd>(
+pub fn aes_ctr_tmp<Fd: AsFd>(
     setup_fds: (RawFd, RawFd),
-    fd: &F,
+    fd: Fd,
     flags: OFlag,
     tmp: Option<RawFd>,
     restrict_memfd: bool,
-) -> Result<Option<(RawFd, IV)>, Errno> {
+) -> Result<Option<(OwnedFd, IV)>, Errno> {
     let (aes_fd, mac_fd) = setup_fds;
 
     // Check if this is a Syd encrypted file.
     #[allow(clippy::cast_possible_truncation)]
     #[allow(clippy::cast_sign_loss)]
-    let size = lseek64(fd.as_fd().as_raw_fd(), 0, Whence::SeekEnd)? as usize;
+    let size = lseek64(&fd, 0, Whence::SeekEnd)? as usize;
     #[allow(clippy::arithmetic_side_effects)]
     let iv_and_tag = if size == 0 {
         // Encrypting new file.
@@ -612,12 +712,12 @@ pub fn aes_ctr_tmp<F: AsFd>(
         return Ok(None);
     } else {
         // Read and verify file magic.
-        lseek64(fd.as_fd().as_raw_fd(), 0, Whence::SeekSet)?;
+        lseek64(&fd, 0, Whence::SeekSet)?;
         let mut magic = [0u8; CRYPT_MAGIC.len()];
         let mut nread = 0;
         while nread < magic.len() {
             #[allow(clippy::arithmetic_side_effects)]
-            match read(fd.as_fd().as_raw_fd(), &mut magic[nread..]) {
+            match read(&fd, &mut magic[nread..]) {
                 Ok(0) => {
                     // SAFETY: Not a Syd file, do nothing.
                     return Ok(None);
@@ -639,7 +739,7 @@ pub fn aes_ctr_tmp<F: AsFd>(
         let mut nread = 0;
         while nread < buf.len() {
             #[allow(clippy::arithmetic_side_effects)]
-            match read(fd.as_fd().as_raw_fd(), &mut buf[nread..]) {
+            match read(&fd, &mut buf[nread..]) {
                 Ok(0) => {
                     // SAFETY: Corrupt HMAC tag, return error.
                     return Err(Errno::EBADMSG);
@@ -657,7 +757,7 @@ pub fn aes_ctr_tmp<F: AsFd>(
         let mut nread = 0;
         while nread < buf.len() {
             #[allow(clippy::arithmetic_side_effects)]
-            match read(fd.as_fd().as_raw_fd(), &mut buf[nread..]) {
+            match read(&fd, &mut buf[nread..]) {
                 Ok(0) => {
                     // SAFETY: Corrupt IV, return error.
                     return Err(Errno::EBADMSG);
@@ -674,10 +774,10 @@ pub fn aes_ctr_tmp<F: AsFd>(
     let dst_fd = if let Some(tmp) = tmp {
         // SAFETY: `tmp' is alive for the duration of the Syd sandbox.
         let tmp = unsafe { BorrowedFd::borrow_raw(tmp) };
-        mkstempat(&tmp, XPath::from_bytes(b""))
+        mkstempat(tmp, XPath::from_bytes(b""))
     } else {
         // Note, MFD_ALLOW_SEALING is implied for MFD_NOEXEC_SEAL.
-        create_memfd_raw(
+        create_memfd(
             b"syd-aes\0",
             if restrict_memfd {
                 MFD_NOEXEC_SEAL
@@ -685,11 +785,7 @@ pub fn aes_ctr_tmp<F: AsFd>(
                 MFD_ALLOW_SEALING
             },
         )
-    }
-    .map(|fd| {
-        // SAFETY: syscall returned success, we have a valid FD.
-        unsafe { BorrowedFd::borrow_raw(fd) }
-    })?;
+    }?;
 
     let iv = if let Some((iv, hmac_tag)) = iv_and_tag {
         // Initialize HMAC socket and feed magic header and IV.
@@ -716,7 +812,7 @@ pub fn aes_ctr_tmp<F: AsFd>(
 
             let n = retry_on_eintr(|| {
                 splice(
-                    fd,
+                    &fd,
                     None,
                     &pipe_wr_dec,
                     None,
@@ -806,7 +902,7 @@ pub fn aes_ctr_tmp<F: AsFd>(
                         splice(
                             &pipe_rd_dec,
                             None,
-                            dst_fd,
+                            &dst_fd,
                             None,
                             ncopy,
                             SpliceFFlags::empty(),
@@ -849,7 +945,7 @@ pub fn aes_ctr_tmp<F: AsFd>(
                     splice(
                         &pipe_rd_dec,
                         None,
-                        dst_fd,
+                        &dst_fd,
                         None,
                         ncopy,
                         SpliceFFlags::empty(),
@@ -882,7 +978,7 @@ pub fn aes_ctr_tmp<F: AsFd>(
     if flags.contains(OFlag::O_APPEND) {
         set_append(&dst_fd, true)?
     } else if size > 0 {
-        lseek64(dst_fd.as_raw_fd(), 0, Whence::SeekSet)?;
+        lseek64(&dst_fd, 0, Whence::SeekSet)?;
     }
 
     // Set non-blocking as necessary.
@@ -890,16 +986,16 @@ pub fn aes_ctr_tmp<F: AsFd>(
         set_nonblock(&dst_fd, true)?;
     }
 
-    Ok(Some((dst_fd.as_raw_fd(), iv)))
+    Ok(Some((dst_fd, iv)))
 }
 
 /// Feed data into the AF_ALG socket from the given file descriptor.
-pub fn aes_ctr_feed<S: AsFd, F: AsFd>(sock: &S, fd: &F, buf: &mut [u8]) -> Result<usize, Errno> {
+pub fn aes_ctr_feed<S: AsFd, F: AsFd>(sock: S, fd: F, buf: &mut [u8]) -> Result<usize, Errno> {
     // Read from the file descriptor.
     let mut nread = 0;
     while nread < buf.len() {
         #[allow(clippy::arithmetic_side_effects)]
-        match read(fd.as_fd().as_raw_fd(), &mut buf[nread..]) {
+        match read(&fd, &mut buf[nread..]) {
             Ok(0) => break, // EOF
             Ok(n) => nread += n,
             Err(Errno::EINTR) => continue,
@@ -924,8 +1020,8 @@ pub fn aes_ctr_feed<S: AsFd, F: AsFd>(sock: &S, fd: &F, buf: &mut [u8]) -> Resul
 
 /// Flush data in the AF_ALG socket into the given file descriptor.
 pub fn aes_ctr_flush<S: AsFd, F: AsFd>(
-    sock: &S,
-    fd: &F,
+    sock: S,
+    fd: F,
     buf: &mut [u8],
     size: usize,
 ) -> Result<usize, Errno> {
@@ -935,7 +1031,7 @@ pub fn aes_ctr_flush<S: AsFd, F: AsFd>(
     let mut nread = 0;
     while nread < size {
         #[allow(clippy::arithmetic_side_effects)]
-        match read(sock.as_fd().as_raw_fd(), &mut buf[nread..size]) {
+        match read(&sock, &mut buf[nread..size]) {
             Ok(0) => return Err(Errno::EINVAL),
             Ok(n) => nread += n,
             Err(Errno::EINTR) => continue,
@@ -947,7 +1043,7 @@ pub fn aes_ctr_flush<S: AsFd, F: AsFd>(
     let mut nwrite = 0;
     while nwrite < nread {
         #[allow(clippy::arithmetic_side_effects)]
-        match write(fd, &buf[nwrite..nread]) {
+        match write(&fd, &buf[nwrite..nread]) {
             Ok(0) => return Err(Errno::EINVAL),
             Ok(n) => nwrite += n,
             Err(Errno::EINTR) => continue,
@@ -1039,6 +1135,56 @@ pub fn get_at_random_name(idx: usize) -> String {
     }
 }
 
+/// SydRandomState: a `BuildHasher` that seeds `AHasher`
+/// with 256 bits of OS entropy using `syd::fs::getrandom`,
+/// aka getentropy(3).
+#[derive(Clone, Copy, Debug, PartialEq, Eq)]
+pub struct SydRandomState {
+    k0: u64,
+    k1: u64,
+    k2: u64,
+    k3: u64,
+}
+
+impl SydRandomState {
+    /// Grab 32 bytes from OS RNG with getentropy(3),
+    /// split into four u64 seeds.
+    #[inline]
+    #[allow(clippy::disallowed_methods)]
+    pub fn new() -> Self {
+        // Pull 32 bytes (4 x 8) from OS RNG.
+        // Panics if entropy cannot be fetched.
+        let mut buf = [0u8; 32];
+        fillrandom(&mut buf).expect("SydRandomState: failed to acquire 32 bytes of entropy");
+
+        // Safety: We know `buf` is exactly 32 bytes long,
+        // so slicing into four 8-byte chunks is always valid.
+        let k0 = u64::from_ne_bytes(buf[0..8].try_into().unwrap());
+        let k1 = u64::from_ne_bytes(buf[8..16].try_into().unwrap());
+        let k2 = u64::from_ne_bytes(buf[16..24].try_into().unwrap());
+        let k3 = u64::from_ne_bytes(buf[24..32].try_into().unwrap());
+
+        SydRandomState { k0, k1, k2, k3 }
+    }
+}
+
+impl Default for SydRandomState {
+    #[inline]
+    fn default() -> Self {
+        Self::new()
+    }
+}
+
+impl BuildHasher for SydRandomState {
+    type Hasher = AHasher;
+
+    #[inline]
+    #[allow(clippy::disallowed_types)]
+    fn build_hasher(&self) -> Self::Hasher {
+        RandomState::with_seeds(self.k0, self.k1, self.k2, self.k3).build_hasher()
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use std::io::Cursor;
@@ -1892,7 +2038,7 @@ mod tests {
             (sock_dec.as_raw_fd(), sock_mac.as_raw_fd()),
             &encrypted_memfd,
             OFlag::empty(),
-            Some(tmp_dir),
+            Some(tmp_dir.as_raw_fd()),
             false,
         ) {
             Ok(fd) => fd.unwrap(),
@@ -1909,12 +2055,12 @@ mod tests {
         // Verify the decrypted data matches the original data.
         let mut decrypted_data = vec![0u8; total_size];
         lseek64(
-            decrypted_memfd.as_raw_fd(),
+            &decrypted_memfd,
             (CRYPT_MAGIC.len() + IV_SIZE) as i64,
             Whence::SeekSet,
         )
         .unwrap();
-        read(decrypted_memfd.as_raw_fd(), &mut decrypted_data).unwrap();
+        read(decrypted_memfd, &mut decrypted_data).unwrap();
         assert_eq!(
             decrypted_data,
             data,
