@@ -10,10 +10,10 @@ use libseccomp::ScmpNotifResp;
 use nix::errno::Errno;
 
 use crate::{
-    compat::{fstatx, statx, STATX_INO, STATX_MNT_ID, STATX_MNT_ID_UNIQUE},
-    config::{HAVE_STATX_MNT_ID_UNIQUE, PROC_FILE},
+    compat::{fstatx, statx},
+    config::PROC_FILE,
     debug, error,
-    fs::{readlinkat, CanonicalPath, FsFlags},
+    fs::{readlinkat, CanonicalPath, FileInfo, FsFlags},
     hook::{PathArgs, RemoteProcess, SysArg, SysFlags, UNotifyEventRequest},
     kernel::{sandbox_path, syscall_path_handler},
     path::XPathBuf,
@@ -103,17 +103,10 @@ pub(crate) fn sysexit_chdir(
     let mut pfd = XPathBuf::from_pid(process.pid);
     pfd.push(b"cwd");
 
-    let mut mask = STATX_INO;
-    mask |= if *HAVE_STATX_MNT_ID_UNIQUE {
-        STATX_MNT_ID_UNIQUE
-    } else {
-        STATX_MNT_ID
-    };
-
     #[allow(clippy::disallowed_methods)]
     let fd = path.dir.as_ref().unwrap();
 
-    let stx_fd = match fstatx(fd, mask) {
+    let stx_fd = match fstatx(fd, FileInfo::mask()) {
         Ok(stx) => stx,
         Err(errno) => {
             // SAFETY: Failed to stat FD,
@@ -128,7 +121,7 @@ pub(crate) fn sysexit_chdir(
         }
     };
 
-    let stx_cwd = match statx(PROC_FILE(), &pfd, 0, mask) {
+    let stx_cwd = match statx(PROC_FILE(), &pfd, 0, FileInfo::mask()) {
         Ok(stx) => stx,
         Err(errno) => {
             // SAFETY: Failed to stat CWD,
