@@ -6,15 +6,13 @@
 //
 // SPDX-License-Identifier: GPL-3.0
 
-use std::{
-    ffi::CStr,
-    os::fd::{FromRawFd, OwnedFd, RawFd},
-};
+use std::ffi::CStr;
 
 use libseccomp::ScmpNotifResp;
 use nix::errno::Errno;
 
 use crate::{
+    cookie::safe_memfd_create,
     fs::{MFD_CLOEXEC, MFD_EXEC, MFD_NOEXEC_SEAL},
     hook::{RemoteProcess, UNotifyEventRequest},
     kernel::sandbox_path,
@@ -39,6 +37,8 @@ pub(crate) fn sys_memfd_create(request: UNotifyEventRequest) -> ScmpNotifResp {
         // If sandboxing for create capability is off, return immediately.
         let sandbox = request.get_sandbox();
         let check = sandbox.enabled(Capability::CAP_CREATE);
+        let force_cloexec = sandbox.force_cloexec();
+        let force_rand_fd = sandbox.force_rand_fd();
         let restrict_memfd = !sandbox.allow_unsafe_memfd();
 
         #[allow(clippy::cast_possible_truncation)]
@@ -86,23 +86,18 @@ pub(crate) fn sys_memfd_create(request: UNotifyEventRequest) -> ScmpNotifResp {
 
         // Set CLOEXEC for our fd always, and
         // Set CLOEXEC for remote fd as necessary.
-        let cloexec = flags & MFD_CLOEXEC != 0;
+        let cloexec = force_cloexec || (flags & MFD_CLOEXEC != 0);
         flags |= MFD_CLOEXEC;
 
         // Access granted, emulate call and return the fd to the process.
-        // SAFETY: nix does not support all flags we want to pass toe memfd_create(2).
-        let fd = match Errno::result(unsafe { libc::memfd_create(name.as_ptr(), flags) }) {
-            Ok(fd) => {
-                // SAFETY: memfd_create returns a valid FD.
-                unsafe { OwnedFd::from_raw_fd(fd as RawFd) }
-            }
+        match safe_memfd_create(name.to_bytes_with_nul(), flags) {
+            Ok(fd) => request.send_fd(fd, cloexec, force_rand_fd),
             Err(Errno::EINVAL) => {
                 // Return ENOSYS rather than EINVAL if MFD_NOEXEC_SEAL is unsupported.
                 // FIXME: This case includes other invalid flag combinations!
-                return Err(Errno::ENOSYS);
+                Err(Errno::ENOSYS)
             }
-            Err(errno) => return Err(errno),
-        };
-        request.send_fd(fd, cloexec)
+            Err(errno) => Err(errno),
+        }
     })
 }
