@@ -17,7 +17,7 @@ use nix::{
 use crate::{
     fs::FsFlags,
     hook::{PathArgs, SysArg, UNotifyEventRequest},
-    kernel::syscall_path_handler,
+    kernel::{syscall_path_handler, to_mode, to_sflag},
     proc::proc_umask,
 };
 
@@ -143,27 +143,3 @@ fn syscall_mknod_handler(
     )
     .map(|_| request.return_syscall(0))
 }
-
-#[inline]
-fn to_sflag(arg: u64) -> Result<SFlag, Errno> {
-    let kind = arg
-        .try_into()
-        .map(|kind: libc::mode_t| kind & SFlag::S_IFMT.bits())
-        .or(Err(Errno::EINVAL))?;
-
-    // Careful here, zero file type is equivalent to S_IFREG.
-    if kind == 0 {
-        Ok(SFlag::S_IFREG)
-    } else {
-        SFlag::from_bits(kind).ok_or(Errno::EINVAL)
-    }
-}
-
-#[inline]
-fn to_mode(arg: u64) -> Result<Mode, Errno> {
-    let mode = arg
-        .try_into()
-        .map(|mode: libc::mode_t| mode & !SFlag::S_IFMT.bits())
-        .or(Err(Errno::EINVAL))?;
-    Mode::from_bits(mode).ok_or(Errno::EINVAL)
-}
