@@ -130,7 +130,11 @@ use std::{borrow::Cow, os::fd::RawFd};
 
 use libseccomp::ScmpNotifResp;
 use memchr::memmem;
-use nix::{errno::Errno, fcntl::AtFlags, sys::stat::Mode};
+use nix::{
+    errno::Errno,
+    fcntl::AtFlags,
+    sys::stat::{Mode, SFlag},
+};
 
 use crate::{
     fs::{CanonicalPath, FileInfo, FileType},
@@ -585,11 +589,29 @@ pub(crate) fn to_atflags(arg: u64, valid: AtFlags) -> Result<AtFlags, Errno> {
 }
 
 #[inline]
-fn to_mode(arg: u64) -> Result<Mode, Errno> {
-    let mode = arg.try_into().or(Err(Errno::EINVAL))?;
+pub(crate) fn to_mode(arg: u64) -> Result<Mode, Errno> {
+    let mode = arg
+        .try_into()
+        .map(|mode: libc::mode_t| mode & !SFlag::S_IFMT.bits())
+        .or(Err(Errno::EINVAL))?;
     Mode::from_bits(mode).ok_or(Errno::EINVAL)
 }
 
+#[inline]
+pub(crate) fn to_sflag(arg: u64) -> Result<SFlag, Errno> {
+    let kind = arg
+        .try_into()
+        .map(|kind: libc::mode_t| kind & SFlag::S_IFMT.bits())
+        .or(Err(Errno::EINVAL))?;
+
+    // Careful here, zero file type is equivalent to S_IFREG.
+    if kind == 0 {
+        Ok(SFlag::S_IFREG)
+    } else {
+        SFlag::from_bits(kind).ok_or(Errno::EINVAL)
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
