@@ -218,11 +218,13 @@ fn syscall_network_handler(request: UNotifyEventRequest, args: &[u64; 6], op: u8
                     // SAFETY: allow_unsupp_socket:1
                     // Safe domain, allow.
                 }
+                let force_cloexec = sandbox.force_cloexec();
+                let force_rand_fd = sandbox.force_rand_fd();
                 drop(sandbox); // drop the read-lock before emulation.
 
                 let stype = libc::c_int::try_from(args[1]).or(Err(Errno::EINVAL))?;
                 let proto = libc::c_int::try_from(args[2]).or(Err(Errno::EAFNOSUPPORT))?;
-                let cloexec = stype & libc::SOCK_CLOEXEC != 0;
+                let cloexec = force_cloexec || (stype & libc::SOCK_CLOEXEC != 0);
                 let stype = stype | libc::SOCK_CLOEXEC;
 
                 let fd =
@@ -255,7 +257,7 @@ fn syscall_network_handler(request: UNotifyEventRequest, args: &[u64; 6], op: u8
                         "inode": inode);
                 }
 
-                return request.send_fd(fd, cloexec);
+                return request.send_fd(fd, cloexec, force_rand_fd);
             }
             0x2 | 0x6 => Capability::CAP_NET_BIND,
             _ => Capability::CAP_NET_CONNECT,
