@@ -13,17 +13,17 @@ use std::{
     os::fd::{AsFd, AsRawFd, IntoRawFd, OwnedFd, RawFd},
 };
 
-use bitflags::Flags;
 use libseccomp::ScmpNotifResp;
 use nix::{
     errno::Errno,
-    fcntl::{openat2, OFlag, OpenHow, ResolveFlag},
+    fcntl::{OFlag, OpenHow, ResolveFlag},
     sys::stat::{umask, Mode},
     NixPath,
 };
 
 use crate::{
     config::{AES_CYCLE_TIME, ESYD_SH, MAGIC_PREFIX, NULL_FD, PROC_FILE, ROOT_FILE, SYD_EL},
+    cookie::safe_openat2,
     fs::{
         create_memfd, lock_fd, safe_open_how_magicsym, safe_open_magicsym, seal_memfd, unlock_fd,
         CanonicalPath, FileInfo, FileType, FsFlags, MaybeFd, MFD_ALLOW_SEALING,
@@ -32,14 +32,14 @@ use crate::{
     hook::{SysArg, UNotifyEventRequest},
     info, log_enabled,
     path::{XPath, XPathBuf},
-    pool::{AesMod, AesVal},
     proc::{proc_tty, proc_umask},
     sandbox::{Action, Capability},
     syslog::LogLevel,
     warn,
+    workers::aes::{AesMod, AesVal},
 };
 
-// `OpenType` represents possible open family system calls.
+// `OpenSyscall` represents possible open family system calls.
 //
 //  The list of open family system calls are: creat(2), open(2),
 //  openat(2), and openat2(2).
@@ -82,8 +82,8 @@ pub(crate) fn sys_creat(request: UNotifyEventRequest) -> ScmpNotifResp {
 pub(crate) fn sys_open(request: UNotifyEventRequest) -> ScmpNotifResp {
     let req = request.scmpreq;
 
-    // SAFETY: Reject undefined/invalid flags.
-    let flags = match to_oflag(req.data.args[1]) {
+    // SAFETY: Truncate undefined/invalid flags.
+    let flags = match to_oflag(req.data.args[1], true) {
         Ok(flags) => flags,
         Err(errno) => return request.fail_syscall(errno),
     };
@@ -103,8 +103,8 @@ pub(crate) fn sys_open(request: UNotifyEventRequest) -> ScmpNotifResp {
 pub(crate) fn sys_openat(request: UNotifyEventRequest) -> ScmpNotifResp {
     let req = request.scmpreq;
 
-    // SAFETY: Reject undefined/invalid flags.
-    let flags = match to_oflag(req.data.args[2]) {
+    // SAFETY: Truncate undefined/invalid flags.
+    let flags = match to_oflag(req.data.args[2], true) {
         Ok(flags) => flags,
         Err(errno) => return request.fail_syscall(errno),
     };
@@ -133,7 +133,7 @@ pub(crate) fn sys_openat2(request: UNotifyEventRequest) -> ScmpNotifResp {
     };
 
     // SAFETY: Reject undefined and invalid flags.
-    let flags = match to_oflag(open_how.flags) {
+    let flags = match to_oflag(open_how.flags, false) {
         Ok(flags) => flags,
         Err(errno) => return request.fail_syscall(errno),
     };
@@ -173,10 +173,12 @@ fn syscall_open_handler(
         // descriptor with O_CLOEXEC flag for safety. Here, we mark the
         // state of the O_CLOEXEC flag to use it in seccomp_addfd call.
         // Mutability:
-        //   1. flags is mutable because trace/allow_unsafe_open:0
+        //   1. o_cloexec is mutable because trace/open_force_cloexec:1
+        //      may change its value for O_CLOEXEC.
+        //   2. flags is mutable because trace/allow_unsafe_open:0
         //      may change its value for O_PATH. Append-only may
         //      also edit flags.
-        let o_cloexec = flags.contains(OFlag::O_CLOEXEC);
+        let mut o_cloexec = flags.contains(OFlag::O_CLOEXEC);
         let mut flags = flags | OFlag::O_CLOEXEC;
 
         // Determine capabilities based on access mode:
@@ -241,6 +243,12 @@ fn syscall_open_handler(
             caps.insert(Capability::CAP_TRUNCATE);
         }
 
+        let has_write = caps.intersects(Capability::CAP_WRSET);
+        if has_write && sandbox.force_ro_open() {
+            // SAFETY: Deny write attempt with trace/force_ro_open:1.
+            return Err(Errno::EACCES);
+        }
+
         if sandbox.verbose {
             crate::debug!("ctx": "open", "op": "init",
                 "cap": caps,
@@ -317,7 +325,8 @@ fn syscall_open_handler(
             };
 
             // Send the file descriptor to the process and return the fd no.
-            return request.send_fd(fd, true);
+            // SAFETY: Randomize FD to prevent reuse.
+            return request.send_fd(fd, true, true);
         }
 
         // Validate file flags based on file type information.
@@ -439,7 +448,6 @@ fn syscall_open_handler(
         };
 
         // Sandboxing.
-        let has_write = caps.intersects(Capability::CAP_WRSET);
         let orig_caps = caps;
         let caps = sandbox.getcaps(orig_caps); // Get enabled caps.
 
@@ -504,8 +512,14 @@ fn syscall_open_handler(
             // 2. Check for masked path and change path to /dev/null.
             // 3. Check for append-only path and edit flags argument.
             // We perform these check only if we're allowing.
-            let crypted = if sandbox.is_masked(&path_check) {
-                path = CanonicalPath::new_null();
+            let crypted = if let Some(mask) = sandbox.is_masked(&path_check) {
+                match mask {
+                    None => path = CanonicalPath::new_null(),
+                    Some(mask) => path = CanonicalPath::new_mask(mask, &path_check)?,
+                };
+                // SAFETY: Mask path is not subject to sandbox check!
+                // `path_check' is only used for logging and points
+                // to the original path.
                 path_check = Cow::Borrowed(path.abs());
                 false // masked path is not encrypted.
             } else if sandbox.enabled(Capability::CAP_CRYPT) {
@@ -547,6 +561,10 @@ fn syscall_open_handler(
         let unsafe_open_path = sandbox.allow_unsafe_open_path();
         let unsafe_open_cdev = sandbox.allow_unsafe_open_cdev();
         let restrict_memfd = !sandbox.allow_unsafe_memfd();
+        if sandbox.force_cloexec() {
+            o_cloexec = true;
+        }
+        let o_rand_fd = sandbox.force_rand_fd();
         let setup_fds = sandbox.crypt_setup();
         let crypt_tmp = if crypted {
             sandbox.crypt_tmp.as_ref().map(|fd| fd.as_raw_fd())
@@ -750,6 +768,7 @@ fn syscall_open_handler(
                         mode,
                         force_umask,
                         o_cloexec,
+                        o_rand_fd,
                         restrict_memfd,
                     )? {
                         // read-only encryption.
@@ -786,8 +805,7 @@ fn syscall_open_handler(
                         request.cache.add_sys_block(req, false)?;
                     }
 
-                    #[allow(clippy::disallowed_methods)]
-                    let result = openat2(PROC_FILE(), &pfd, how);
+                    let result = safe_openat2(PROC_FILE(), &pfd, how);
 
                     // Remove invalidation record unless interrupted.
                     if is_blocking && !matches!(result, Err(Errno::EINTR)) {
@@ -815,8 +833,7 @@ fn syscall_open_handler(
                     }
 
                     // All set, open the file.
-                    #[allow(clippy::disallowed_methods)]
-                    let result = openat2(fd, path.base, how);
+                    let result = safe_openat2(fd, path.base, how);
 
                     // Remove invalidation record unless interrupted.
                     if is_blocking && !matches!(result, Err(Errno::EINTR)) {
@@ -827,7 +844,7 @@ fn syscall_open_handler(
                 };
 
                 // File opened successfully, return to caller at one go.
-                request.send_fd(fd, o_cloexec)
+                request.send_fd(fd, o_cloexec, o_rand_fd)
             }
             Action::Deny | Action::Filter if hidden => Err(Errno::ENOENT),
             Action::Deny | Action::Filter => Err(Errno::EACCES),
@@ -843,7 +860,6 @@ fn syscall_open_handler(
 }
 
 /// Handle open() calls.
-#[allow(clippy::disallowed_methods)]
 fn do_open(path: &CanonicalPath, mut safe_flags: OFlag) -> Result<OwnedFd, Errno> {
     if path.base.is_empty() {
         // Existing path: Construct path to proc magic symlink.
@@ -879,7 +895,7 @@ fn do_open(path: &CanonicalPath, mut safe_flags: OFlag) -> Result<OwnedFd, Errno
             )
         };
 
-        openat2(fd, base, how)
+        safe_openat2(fd, base, how)
     }
 }
 
@@ -894,6 +910,7 @@ fn handle_crypt(
     mode: Mode,
     force_umask: Option<Mode>,
     o_cloexec: bool,
+    o_rand_fd: bool,
     restrict_memfd: bool,
 ) -> Result<Option<OwnedFd>, Errno> {
     let my_mode = AesMod::from(safe_flags);
@@ -984,6 +1001,7 @@ fn handle_crypt(
                 mode,
                 force_umask,
                 o_cloexec,
+                o_rand_fd,
                 restrict_memfd,
             );
         }
@@ -1033,7 +1051,7 @@ fn handle_crypt(
     // No need to wait on this lock, as we've just opened the file.
     // Close the send fd to get rid off our copy of the lock.
     lock_fd(&aes_fd, false, false)?;
-    request.send_fd(aes_fd, o_cloexec)?;
+    request.send_fd(aes_fd, o_cloexec, o_rand_fd)?;
 
     // Record encryption process information.
     {
@@ -1058,7 +1076,6 @@ fn handle_crypt(
 }
 
 /// Handle open() calls that can potentially create files.
-#[allow(clippy::disallowed_methods)]
 fn handle_creat(
     request: &UNotifyEventRequest,
     path: &CanonicalPath,
@@ -1086,56 +1103,47 @@ fn handle_creat(
 
     // All set, make the open call.
     let fd = path.dir.as_ref().map(|fd| fd.as_fd()).ok_or(Errno::EBADF)?;
-    openat2(fd, path.base, how)
+    safe_openat2(fd, path.base, how)
 }
 
-fn to_oflag(arg: u64) -> Result<OFlag, Errno> {
+fn to_oflag(arg: u64, truncate: bool) -> Result<OFlag, Errno> {
     // SAFETY: Reject undefined flags.
     let flags = arg.try_into().or(Err(Errno::EINVAL))?;
 
     // SAFETY: Reject invalid flags.
-    //
-    const KEEP_OFLAG: &[OFlag] = &[
+    const KEEP_OFLAG: OFlag =
         // Keep O_LARGEFILE for old ABI.
-        OFlag::from_bits_retain(0x8000),
-    ];
-    let mut have_oflag = [false];
+        OFlag::from_bits_retain(0x8000);
 
     let mut flags = OFlag::from_bits_retain(flags);
-    for (idx, flag) in KEEP_OFLAG.iter().enumerate() {
-        if flags.contains(*flag) {
-            have_oflag[idx] = true;
-            flags.remove(*flag);
-        }
+    let preserved = flags & KEEP_OFLAG;
+    flags.remove(KEEP_OFLAG);
+
+    if truncate {
+        // open, openat truncates invalid flags.
+        return Ok((flags & OFlag::all()) | preserved);
     }
 
-    if flags.contains_unknown_bits() {
-        let unknown_flags = flags & !OFlag::all();
+    // openat2 validates invalid flags.
+    let unknown_flags = flags.difference(OFlag::all());
+    if unknown_flags.bits() != 0 {
         crate::error!("ctx": "open", "op": "invalid_flags",
             "msg": "passed invalid flags to open flags argument",
             "flags": format!("{unknown_flags:?}"));
         return Err(Errno::EINVAL);
     }
 
-    flags |= KEEP_OFLAG
-        .iter()
-        .zip(&have_oflag)
-        .filter(|&(_, &keep)| keep)
-        .map(|(flag, _)| *flag)
-        .fold(OFlag::empty(), |acc, f| acc | f);
-
-    Ok(flags)
+    Ok(flags | preserved)
 }
 
+#[inline]
 fn to_mode(arg: u64, flags: OFlag) -> Result<Mode, Errno> {
-    let mode = if flags.contains(OFlag::O_CREAT) || flags.contains(OFlag::O_TMPFILE) {
+    if flags.contains(OFlag::O_CREAT) || flags.contains(OFlag::O_TMPFILE) {
         // SAFETY: Reject invalid mode with creation flags.
-        arg.try_into().or(Err(Errno::EINVAL))?
+        crate::kernel::to_mode(arg)
     } else {
         // SAFETY: mode argument is ignored without creation flags.
         // glibc sets it to 0, musl does not. Both are valid behaviour.
-        0
-    };
-
-    Mode::from_bits(mode).ok_or(Errno::EINVAL)
+        Ok(Mode::empty())
+    }
 }
