@@ -109,9 +109,13 @@
 // TODO: remove the allow-lint below when libseccomp is back to upstream.
 #![allow(macro_expanded_macro_exports_accessed_by_absolute_paths)]
 
+/// System call argument cookies
+pub(crate) mod cookie;
 /// System call handlers
 #[macro_use]
 pub(crate) mod kernel;
+/// Worker threads
+pub(crate) mod workers;
 
 /// Utilities for caching
 pub mod cache;
@@ -194,11 +198,6 @@ extern crate lazy_static;
 #[allow(clippy::type_complexity)]
 #[allow(clippy::undocumented_unsafe_blocks)]
 pub mod landlock;
-#[allow(dead_code)]
-#[allow(clippy::disallowed_methods)]
-#[allow(clippy::manual_flatten)]
-#[allow(clippy::too_many_arguments)]
-#[allow(missing_docs)]
 /// rusty_pool: Self growing / shrinking `ThreadPool` implementation
 pub(crate) mod pool;
 /// The low-level interface for linux namespaces (containers)
