@@ -464,8 +464,8 @@ pub fn log_init(_default_level: LogLevel, default_log_fd: Option<RawFd>) -> Resu
     if !tty {
         if std::env::var_os(ENV_LOG_FD).is_none() && std::env::var_os(ENV_QUIET_TTY).is_none() {
             // check if we have a TTY
-            tty = nix::unistd::isatty(libc::STDIN_FILENO).unwrap_or(false)
-                && nix::unistd::isatty(libc::STDERR_FILENO).unwrap_or(false);
+            tty = nix::unistd::isatty(std::io::stdin()).unwrap_or(false)
+                && nix::unistd::isatty(std::io::stderr()).unwrap_or(false);
         } else {
             tty = false;
         }
