@@ -16,9 +16,11 @@
 //
 // SPDX-License-Identifier: GPL-3.0
 
-// SAFETY: This module has been liberated from unsafe code!
+// SAFETY: This module has (almost) been liberated from unsafe code!
+// proc_rand_fd uses kcmp(2) which calls into libc therefore needs unsafe.
 // proc_set_at_secure_* tests call fork(2) therefore they need unsafe.
-#![cfg_attr(not(test), forbid(unsafe_code))]
+// Use deny rather than forbid so we can allow those cases.
+#![deny(unsafe_code)]
 
 use std::{
     collections::{HashMap, HashSet},
@@ -47,7 +49,7 @@ use nix::{
     unistd::{getpid, lseek64, read, Gid, Pid, Uid, Whence},
 };
 use procfs::{
-    process::{MMPermissions, MMapPath, MemoryMaps},
+    process::{LimitValue, MMPermissions, MMapPath, MemoryMaps},
     FromBufRead,
 };
 
@@ -56,11 +58,11 @@ use crate::{
     config::*,
     elf::ElfType,
     err::err2no,
-    fs::{parse_fd, safe_open},
+    fs::{parse_fd, randint, safe_open},
     hash::SydRandomState,
     parsers::{
         map_result,
-        proc::{parse_stat, parse_statm, parse_status},
+        proc::{parse_max_open_files, parse_stat, parse_statm, parse_status},
         read_to_end,
     },
     path::XPathBuf,
@@ -1214,6 +1216,73 @@ pub fn proc_pid_fd(pid: Option<Pid>) -> Result<Vec<(RawFd, XPathBuf)>, Errno> {
     Ok(res)
 }
 
+/// Return the max open files soft limit for the given process.
+pub fn proc_max_open_files(pid: Pid) -> Result<LimitValue, Errno> {
+    let mut path = XPathBuf::from_pid(pid);
+    path.push(b"limits");
+
+    let mut file = safe_open(PROC_FILE(), &path, OFLAG_PROC)
+        .map(File::from)
+        .or(Err(Errno::ESRCH))?;
+
+    let mut buf = [0; 1792]; // A typical limits file is about 1500 bytes.
+    map_result(parse_max_open_files(read_to_end(&mut file, &mut buf)?))
+}
+
+/// Pick a random fd number and check via KCMP_FILE whether it’s open in `pid`.
+/// On success returns the fd; if none are found after probing the full range,
+/// returns `Err(Errno::EBADF)`. Other errors (permissions, invalid pid, etc.)
+/// are returned immediately.
+pub fn proc_rand_fd(pid: Pid) -> Result<RawFd, Errno> {
+    const KCMP_FILE: u64 = 0;
+
+    // SAFETY: Cap to a sane maximum because sufficiently
+    // big values of the hard limit tend to return ENOMEM.
+    const OFILE_MAX: u64 = 0x10000;
+    let range_start = 7u64;
+    let range_end = match proc_max_open_files(pid)? {
+        LimitValue::Unlimited => OFILE_MAX,
+        LimitValue::Value(val) => val.saturating_sub(1).min(OFILE_MAX),
+    };
+
+    if range_end <= range_start {
+        return Err(Errno::EMFILE);
+    }
+    let range = range_start..=range_end;
+
+    // SAFETY: To make this file descriptor harder to spot by an
+    // attacker we duplicate it to a random fd number.
+    for _ in range.clone() {
+        #[allow(clippy::cast_possible_truncation)]
+        let fd_rand = randint(range.clone())? as RawFd;
+
+        // Check if the slot is free.
+        // This is subject to race, and optimally should
+        // be implemented in the host kernel.
+        // SAFETY: There's no libc wrapper for kcmp.
+        #[allow(unsafe_code)]
+        let result = Errno::result(unsafe {
+            libc::syscall(
+                libc::SYS_kcmp,
+                pid.as_raw(),
+                pid.as_raw(),
+                KCMP_FILE,
+                fd_rand,
+                fd_rand,
+            )
+        })
+        .map(drop);
+
+        return match result {
+            Ok(_) => continue,
+            Err(Errno::EBADF) => Ok(fd_rand),
+            Err(errno) => Err(errno),
+        };
+    }
+
+    Err(Errno::EBADF)
+}
+
 /// Log `/proc/self/fd` contents.
 ///
 /// Useful for debugging file descriptor leaks.
@@ -1469,6 +1538,7 @@ mod tests {
         }
 
         // Fork and execve the compiled program under ptrace.
+        #[allow(unsafe_code)]
         match unsafe { fork() } {
             Ok(ForkResult::Child) => {
                 // Child process
