@@ -723,6 +723,10 @@ fn main() -> SydResult<ExitCode> {
         }
     }
 
+    // Disable Speculative Store Bypass mitigations
+    // for trace/allow_unsafe_spec_exec:1
+    let ssb = sandbox.allow_unsafe_spec_exec();
+
     // SAFETY: fork is our safest option here.
     let child = match unsafe { fork() }? {
         ForkResult::Child => {
@@ -773,7 +777,31 @@ fn main() -> SydResult<ExitCode> {
     // 2. waitid and exit.
     // 3. memory allocation syscalls
     // 4. signal handling syscalls
-    let mut filter = ScmpFilterContext::new(ScmpAction::KillProcess)?;
+    let mut ctx = ScmpFilterContext::new(ScmpAction::KillProcess)?;
+
+    // Enforce the NO_NEW_PRIVS functionality before
+    // loading the seccomp filter into the kernel.
+    ctx.set_ctl_nnp(true)?;
+
+    // Disable Speculative Store Bypass mitigations
+    // with trace/allow_unsafe_spec_exec:1
+    ctx.set_ctl_ssb(ssb)?;
+
+    // DO NOT synchronize filter to all threads.
+    // Main thread will confine itself.
+    ctx.set_ctl_tsync(false)?;
+
+    // We kill for bad system call and bad arch.
+    ctx.set_act_badarch(ScmpAction::KillProcess)?;
+
+    // Use a binary tree sorted by syscall number if possible.
+    let _ = ctx.set_ctl_optimize(2);
+
+    // SAFETY: Do NOT add supported architectures to the filter.
+    // This ensures Syd can never run a non-native system call,
+    // which we do not need at all.
+    // seccomp_add_architectures(&mut ctx)?;
+
     let allow_call = [
         "exit",
         "exit_group",
@@ -802,14 +830,14 @@ fn main() -> SydResult<ExitCode> {
     ];
     for name in allow_call {
         if let Ok(syscall) = ScmpSyscall::from_name(name) {
-            filter.add_rule(ScmpAction::Allow, syscall)?;
+            ctx.add_rule(ScmpAction::Allow, syscall)?;
         }
     }
 
     // Allow write to standard error.
     for name in ["write", "writev"] {
         if let Ok(syscall) = ScmpSyscall::from_name(name) {
-            filter.add_rule_conditional(
+            ctx.add_rule_conditional(
                 ScmpAction::Allow,
                 syscall,
                 &[scmp_cmp!($arg0 == libc::STDERR_FILENO as u64)],
@@ -821,14 +849,14 @@ fn main() -> SydResult<ExitCode> {
     const PROT_EXEC: u64 = libc::PROT_EXEC as u64;
     for name in ["mmap", "mmap2", "mprotect"] {
         #[allow(clippy::disallowed_methods)]
-        filter.add_rule_conditional(
+        ctx.add_rule_conditional(
             ScmpAction::Allow,
             ScmpSyscall::from_name(name).unwrap(),
             &[scmp_cmp!($arg2 & PROT_EXEC == 0)],
         )?;
     }
 
-    filter.load()?;
+    ctx.load()?;
 
     loop {
         #[allow(clippy::cast_possible_truncation)]
