@@ -247,7 +247,7 @@ mod syslog_enabled {
         os::fd::{AsRawFd, BorrowedFd},
         sync::{
             atomic::{AtomicBool, AtomicU8, Ordering},
-            Arc, OnceLock, RwLock,
+            OnceLock, RwLock,
         },
     };
 
@@ -259,7 +259,7 @@ mod syslog_enabled {
         storage::{Array, Heap},
         traits::*,
         wrap::caching::Caching,
-        SharedRb,
+        Arc, SharedRb,
     };
 
     use crate::{config::SYSLOG_STACK_SIZE, log::LockedWriter, syslog::*};
