@@ -223,6 +223,13 @@ macro_rules! assert_status_interrupted {
     };
 }
 
+#[macro_export]
+macro_rules! assert_status_unimplemented {
+    ($status:expr) => {
+        assert_status_code!($status, libc::ENOSYS);
+    };
+}
+
 #[macro_export]
 macro_rules! assert_status_killed {
     ($status:expr) => {
