@@ -22,6 +22,7 @@ use crate::{
     caps,
     config::*,
     fs::retry_on_eintr,
+    landlock::{CompatLevel, Compatible, Ruleset, RulesetAttr, Scope},
     safe_drop_cap,
     unshare::{
         error::ErrorCode as Err,
@@ -63,17 +64,40 @@ macro_rules! fail_errno_safe {
 
 #[allow(clippy::cognitive_complexity)]
 pub extern "C" fn child_after_clone(arg: *mut libc::c_void) -> libc::c_int {
+    // SAFETY: Apply a Landlock scope sandbox to restrict
+    // 1. Ptrace attach outside Landlock.
+    // 2. Signal send outside Landlock.
+    // 3. UNIX abstract socket connect outside Landlock.
+    // We leave path and network restrictions for Landlock
+    // to be configured by the user using Lock sandboxing.
+    // This is an added layer for hardening and it's best-effort.
+    let mut ruleset = Ruleset::default();
+    let ruleset_ref = &mut ruleset;
+    ruleset_ref.set_compatibility(CompatLevel::BestEffort);
+    let _ = ruleset_ref.scope(Scope::AbstractUnixSocket);
+    let _ = ruleset_ref.scope(Scope::Signal);
+    if let Ok(ruleset) = ruleset.create() {
+        let _ = ruleset.restrict_self();
+    }
+
     // SAFETY: Do not leak the static file descriptors to the sandbox process.
     proc_close();
 
-    // SAFETY: Do not leak the log file descriptor to the sandbox process.
-    let log_fd = if let Some(fd) = env::var_os(ENV_LOG_FD) {
-        btoi::<RawFd>(fd.as_bytes()).ok()
-    } else {
-        None
-    };
-    if let Some(log_fd) = log_fd {
-        let _ = close(log_fd);
+    // SAFETY: Do not leak the following FDs to the sandbox process:
+    // 1. Log file descriptor.
+    // 2. IPC epoll file descriptor.
+    // 3. IPC UNIX socket descriptor.
+    // TODO: Move this to config.rs.
+    const CLOSE_FD_ENVS: &[&str] = &[ENV_LOG_FD, ENV_IPC_POLL_FD, ENV_IPC_UNIX_FD];
+    for env in CLOSE_FD_ENVS {
+        let fd = if let Some(fd) = env::var_os(env) {
+            btoi::<RawFd>(fd.as_bytes()).ok()
+        } else {
+            None
+        };
+        if let Some(fd) = fd {
+            let _ = close(fd);
+        }
     }
 
     // SAFETY: Clean Syd environment variables from process environment.
