@@ -19,7 +19,7 @@ use nix::{
         signal::{kill, Signal},
         wait::{Id, WaitPidFlag},
     },
-    unistd::{read, setpgid, tcsetpgrp, write, Pid},
+    unistd::{isatty, read, setpgid, tcsetpgrp, write, Pid},
 };
 
 use crate::{
@@ -218,9 +218,30 @@ impl Command {
             cont(pid, Some(Signal::SIGCONT))?;
         }
 
+        let stdin_isatty = isatty(std::io::stdin()).unwrap_or(false);
         if self.config.make_group_leader {
+            // SAFETY: Put sandbox process in a new process group
+            // to isolate Syd processes from the sandbox processes.
             setpgid(pid, pid)?;
-            tcsetpgrp(std::io::stderr(), pid)?;
+
+            // Check if standard input is a TTY.
+            if stdin_isatty {
+                // Set foreground process group to the sandbox process.
+                tcsetpgrp(std::io::stdin(), pid)?;
+            }
+        }
+
+        // SAFETY: Make Syd process the leader of a new process group
+        // to isolate it from the sandbox process. This way signals
+        // sent to sandbox process' process group will not affect
+        // the Syd process.
+        setpgid(Pid::from_raw(0), Pid::from_raw(0))?;
+
+        if stdin_isatty {
+            // SAFETY: Release controlling terminal to ensure
+            // a compromised Syd cannot be used to inject
+            // characters into the controlling terminal.
+            Errno::result(unsafe { libc::ioctl(libc::STDIN_FILENO, libc::TIOCNOTTY) })?;
         }
 
         if let Some(ref mut callback) = self.before_unfreeze {
