@@ -52,12 +52,15 @@ fn main() -> SydResult<ExitCode> {
             println!("- keyring");
             println!("- nice");
             println!("- noop");
+            println!("- page_cache");
+            println!("- perf");
             println!("- pkey");
             println!("- ptrace");
             println!("- safe");
             println!("- setid");
             println!("- time");
             println!("- uring");
+            println!("- uts");
             println!("Given no set, list all files in the current working directory.");
             println!("In this mode, getdents64(2) is used directly.");
             println!("Use to list files in untrusted directories with huge number of files.");
@@ -126,6 +129,11 @@ fn main() -> SydResult<ExitCode> {
                 println!("{name}");
             }
         }
+        Some("page_cache") => {
+            for name in syd::config::PAGE_CACHE_SYSCALLS {
+                println!("{name}");
+            }
+        }
         Some("perf") => {
             for name in syd::config::PERF_SYSCALLS {
                 println!("{name}");
@@ -161,6 +169,11 @@ fn main() -> SydResult<ExitCode> {
                 println!("{name}");
             }
         }
+        Some("uts") => {
+            for name in syd::config::UTS_SYSCALLS {
+                println!("{name}");
+            }
+        }
         Some("env") => {
             for env in syd::config::UNSAFE_ENV {
                 let env = mask_path(Path::new(OsStr::from_bytes(env)));
