@@ -51,11 +51,11 @@ use crate::{
     },
     get_user_home, get_user_name,
     landlock::RulesetStatus,
-    landlock_operation,
+    landlock_policy::LandlockPolicy,
     log::contains_ascii_unprintable,
     path::PATH_MAX,
     unshare::{GidMap, UidMap},
-    xpath, LandlockPolicy, XPath, XPathBuf, MS_NOSYMFOLLOW,
+    xpath, XPath, XPathBuf, MS_NOSYMFOLLOW,
 };
 
 bitflags! {
@@ -377,7 +377,7 @@ impl WordExp<'_> {
 
         // SAFETY: set read end of the pipe as non-blocking.
         let pipe_rd_ref = unsafe { BorrowedFd::borrow_raw(pipe_rd) };
-        set_nonblock(&pipe_rd_ref, true)?;
+        set_nonblock(pipe_rd_ref, true)?;
         // SAFETY: Fork and confine before running wordexp(3)!
 
         let mut stack = [0u8; MINI_STACK_SIZE];
@@ -778,7 +778,7 @@ impl WordExp<'_> {
         };
 
         let abi = crate::landlock::ABI::new_current();
-        match landlock_operation(abi, &policy) {
+        match policy.restrict_self(abi) {
             Ok(status) => match status.ruleset {
                 RulesetStatus::FullyEnforced => {
                     debug!("ctx": "expand",
